\name{Kullback.Leibler}
\alias{Kullback.Leibler}
\title{Multivariate Informationtheoretic Measures}
\description{
  These functions calculate mulivariate informationtheoretic measures: \emph{Kullback Leibler divergence}.
}
\usage{
Kullback.Leibler(p, basis = 2)
}
\arguments{
  \item{p}{ matrix, each column contains probabilities of one distribution across the same alphabet (rows) }
  \item{basis}{ basis of the log used (default=2) }
}
\details{
  \deqn{D_{KL}(P||Q) = \sum(P(x) \dot log \frac{P(x)}{Q(x)})}{D_KL(P||Q) = sum(P*log(P/Q))}
}
\value{
  A square divergence matrix,
  \item{Totals}{always returned: a list with components \emph{H} joint entropy, \emph{Ha} row entropy, \emph{Hab}  row conditional entropy given columns, \emph{Hb} column entropy, \emph{Hba} column conditional entropy given rows, \emph{Im} mutual information }
  \item{Margins}{returned unless grain="Total": \emph{pa} row probabilities , \emph{ha} row entropie, \emph{hab} colwise conditional entropies, \emph{pb} column probabilities, \emph{hb} column entropy , \emph{hba} rowwise conditional entropies}
  \item{Cells}{returned if grain="Cells": \emph{p} joint probabilities, \emph{pab} columnwise conditional probabilities, \emph{pba} rowwise conditional probabilities, \emph{h} joint entropies, \emph{hab} columnwise conditional entropies, \emph{hba} rowwise conditional entropies }
}
\references{ MacKay, David J.C. (2003). \emph{Information Theory, Inference, and Learning Algorithms (chapter 8).} Cambridge University Press.}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{shannon.information}}, \code{\link[distutil]{dist.entropy}}, \code{\link{Kullback.Leibler}}, \code{\link[base]{log}} }
\examples{
 x <- seq(-3, 3, 0.1)
 cp <- pnorm(x)
 p <- cp[-1] - cp[-length(cp)]
 cq <- punif(x, -3, 3)
 q <- cq[-1] - cq[-length(cq)]
 Kullback.Leibler(cbind(p,q))
}
\keyword{ multivariate }
\name{SASmodeclus}
\docType{data}
\alias{SASmodeclus}
\title{Artificial dataset: SASmodeclus}
\description{
One elongated and two convex clusters that can be detected using density based cluster analysis.
}
\usage{data(SASmodeclus)}
\format{A dataframe with columns x, y and cluster}
\source{Similar configuration seen in handbook of commercial software package}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{centerring}}, \code{\link{spiral}}, \code{\link{conflict}}, \code{\link{hierarchic}} }
\keyword{datasets}
\name{align}
\alias{align}
\alias{align.default}
\alias{compare}
\alias{compare.default}
\title{ Cluster aligning/comparing }
\description{
  Aligns or compares objects that are clusterings or contain clusterings
}
\synopsis{
 align(obj, \dots)
 compare(obj, \dots)
}
\usage{
align.default(obj, refobj, method = c("heuristic", "truematch", "tracemax")[1], \dots)
compare.default(obj, method = c("heuristic", "truematch", "tracemax")[1], \dots)
}
\arguments{
  \item{obj}{ object being or containing a clustering }
  \item{refobj}{ an object which defines how to align }
  \item{obj2}{ an object to compare with (columns of table) }
  \item{method}{ one of c("heuristic","truematch","tracemax"), see \code{\link{matchindex}} }
  \item{\dots}{ further arguments depending on class method }
}
\value{
  Function \code{align} returns an aligned object (clustering or object containing a clustering)
  Function \code{compare} returns an aligned table
}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{matchindex}} }

\examples{
 x <- c(1,1,2,2,3,3)
 y <- c(2,2,3,3,1,1)
 table(x, y)
 align(y, x)
 table(x, align(y,x))
 compare(x, y)
}
\keyword{ utilities }
\keyword{ cluster }
\name{centerring}
\docType{data}
\alias{centerring}
\title{Artificial dataset: centerring}
\description{
One spherical cluster surrounded by another ring-shaped cluster. Can be detected by density based cluster algorithms or single link.
}
\usage{data(centerring)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{spiral}}, \code{\link{SASmodeclus}} }
\keyword{datasets}
\name{clusteration}
\alias{clusteration}
\alias{nclusteration}
\alias{print.clusteration}
\title{ Cluster combinations }
\description{
  Functions to construct all possible groupings or calculate the number of possible groupings.
}
\usage{
clusteration(n = NULL, k = NULL, nmax = NULL, kmax = nmax)
nclusteration(n = NULL, k = NULL, nmax = NULL, kmax = nmax)
\method{print}{clusteration}(x, within = "-", between = "+", \dots)
}
\arguments{
  \item{n}{ number of elements }
  \item{k}{ number of groups (optional) }
  \item{nmax}{ maximal number of elements }
  \item{kmax}{ maximal number of groups }
  \item{x}{ an object of class 'clusteration' }
  \item{within}{ character to seperate elements within the same grouping }
  \item{between}{ charachter to seperate between groupings }
  \item{\dots}{ ignored }
}
\details{
  Can be used to create/calculate all groupings (using \code{nmax}), where the number of groupings can be restricted (\code{kmax}).
  The result is a matrix (n x k) which contains in the \code{\link[base]{lower.tri}} Jens' triangle not unlike Pascal's triangle.
  \cr
  Using \code{n} instead of \code{nmax} will return a vector equaling the last line of the triangle,
  using \code{k} can be used to directly return the groupings/number of groupings for a single fixed number of groups.
  All results are created using the triangle construction method
  just calling \code{nclusteration(n=, k=) makes use of Jensen's formula}
  \deqn{C(n,k)=\frac{1}{k!} \cdot \sum_{c=1}^{k}{\over{k}{c} \cdot k^n }}
  The triangle construction method uses the recursive formula
  \deqn{C(n,k)=C(n-1,k-1) + k \cdot C(n-1,k)}
  where \eqn{C(n,1)=1} and \eqn{C(n,n)=1}.
}
\value{
  Function \code{nclusteration} returns either a matrix[n,k] or a vector[k] or a scalar.
  Function \code{clusteration} returns either a list matrix[[n,k]] or a list vector[[k]] or a single element list vector[[1]] of list of \emph{groupings},
  where \emph{groupings} is a list with as many elements as groups and each group beeing represented by a vector of group member indices. A special print method prints this nicely.
}
\references{ Jensen, R.E. (1969) A dynamic programming algorithm for cluster analysis. \emph{Operations Research}, 17, 1034-57. }
\author{ Jens Oehlschlgel }
\note{ The total number of possible groupings increases worse than exponential }
\seealso{ \code{\link{margination}}, \code{\link{combination}}, \code{\link[base]{choose}} }
\examples{
   clusteration(nmax=5)
   clusteration(n=5)
   clusteration(n=5, k=3)
   nclusteration(nmax=5)
   nclusteration(n=5)
   nclusteration(n=5, k=3)

   n <- 100
   k <- 5
   x <- nclusteration(nmax=n)
   matplot(1:n, cbind((1:n)^(1:n), rowSums(x), k^(1:n), x[,k])
   , log="y", type="l"
   , lty=1:4, xlab="n"
   , ylab = "combinations"
   , main = "number of possible groupings"
   )
   legend(n,1
   , c(expression(n^n), expression(k \%in\% group("{", list(1,2,...,n), "}")), substitute(k^n, list(k=k)), substitute(k==c, list(c=k))
   ), lty=1:4, col=1:4, xjust=1, yjust=0)
}
\keyword{ math }
\keyword{ cluster }
\name{combination}
\alias{combination}
\alias{ncombination}
\title{ Combinations }
\description{
  Calculating and creating combinations
}
\usage{
ncombination(k, n, replace = FALSE)
combination(k, n=NULL, x=NULL, replace = FALSE)
}
\arguments{
  \item{k}{ number of positions }
  \item{n}{ poolsize }
  \item{x}{ vector pool of values, can be given instead of n }
  \item{replace}{ default FALSE, TRUE to allow reusing of pool elements }
}
\details{
  \code{ncombination} returns the number of combinations, \code{combination} returns a matrix of combinations. No special sequence granted. Implemented in C, but not faster than the functions in gregmisc.
}
\value{
  A matrix of (k x ncombination)
}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{permutation}}, \code{\link{labelation}}, \code{\link[base]{choose}}, \code{\link[gregmisc]{permutations}} }
\examples{
  ncombination(2,4)
  combination(2,4)
  combination(2,x=letters[1:4])
  ncombination(2,4,replace=TRUE)
  combination(2,4,replace=TRUE)

  \dontrun{
  ## regression test
  require(gregmisc)
  k <- 4
  n <- 6
  replace <- FALSE
  a <- sort(apply(combination(k,n,replace=replace), 2, paste, collapse="-"))
  b <- sort(apply(combinations(n,k,rep=replace), 1, paste, collapse="-"))
  stopifnot(length(a)==ncombination(k,n,replace=replace))
  stopifnot(identical(a,b))
  replace <- TRUE
  a <- sort(apply(combination(k,n,replace=replace), 2, paste, collapse="-"))
  b <- sort(apply(combinations(n,k,rep=replace), 1, paste, collapse="-"))
  stopifnot(length(a)==ncombination(k,n,replace=replace))
  stopifnot(identical(a,b))
  }
}
\keyword{ iteration }
\name{conflict}
\docType{data}
\alias{conflict}
\title{Artificial dataset: conflict}
\description{
This dataset contains two elongated clusters, from a density based perspective. However, from a convex perspective (not sphering the space) we would rather find that 2 halves of those belong together (split in the other dimension).
}
\usage{data(conflict)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{hierarchic}}, \code{\link{noflipper}}, \code{\link{flipper}}, \code{\link{SASmodeclus}}  }
\keyword{datasets}
\name{cophenetic}
\docType{data}
\alias{cophenetic}
\title{Artificial dataset: cophenetic}
\description{
Five cluster with a perfect cophenetic structure: there is a unique way to agglomerate those five. Some procedures fail to recognize five clusters, because 2 of those are very close to each other.
}
\usage{data(cophenetic)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\seealso{ \code{\link{hierarchic}}, \code{\link{noflipper}}, \code{\link{flipper}}, \code{\link{extremegroup}} }
\keyword{datasets}
\name{dist.entropy}
\alias{dist.entropy}
\title{ Entropy distance between variables }
\description{
  This functions calculates the \emph{entropy distance} between variables
}
\usage{
dist.entropy(x)
}
\arguments{
  \item{x}{ a matrix of categorical raw data (cases in rows, variables in columns) }
}
\value{
  An object of class \code{\link[stat]{dist}}.
}
\references{
  MacKay, David J.C. (2003). \emph{Information Theory, Inference, and Learning Algorithms (Exercise 8.5).} Cambridge University Press.
}
\author{ Jens Oehlschlaegel }
\seealso{ \code{\link[stat]{dist}}, \code{\link[rjens]{cor2dist}}, \code{\link[rjens]{cor2sim}}, \code{\link[rjens]{sim2dist}} }
\examples{
  n <- 1000
  x <- runif(n)
  x <- cbind(
    x
  , x
  , 2*x
  , x+runif(n, max=0.1)
  , x+runif(n)
  , x+runif(n, max=10)
  , x+runif(n, max=10)
  , 2*(x+runif(n, max=10))
  , 2*(x+runif(n, max=10))
  , runif(n, max=22)
  , runif(n, max=22)
  )
  x <- round(x)
  colnames(x) <- letters[1:ncol(x)]
  D <- dist.entropy(x)
  plot(hclust(D, method="average"))
}
\keyword{multivariate}
\keyword{cluster}
\name{exact.margin.info}
\alias{exact.margin.info}
\alias{approx.margin.info}
\title{ Univariate "Informatorial" Measures }
\description{
  Functions to evaluate the information within a marginal grouping via complexity-penalized entropy or via the exact combinatorial approach.
}
\usage{
exact.margin.info(...)
approx.margin.info(...)
}
\arguments{
  \item{\dots}{ one or more vectors of group sizes }
}
\details{
  Both functions are sensitive to the number of groups but not to the total number of cases.
}
\value{
  scalar
}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{margination}}, \code{\link{absolute.entropy}}, \code{\link{mmcp2I}} }
\examples{
   require(MASS)
   what <- "compare at fixed n with increasing k"
   cat("\n", what, "\n")
   n <- 1024
   k <- 1:1024
   g <- lapply(k, function(k){g <- rep(n\%/\%k, k);g[0:(n\%\%k)]<-g[0:(n\%\%k)]+1;g})
   c1 <- sapply(g, exact.margin.info)
   c2 <- sapply(g, approx.margin.info)
   eqscplot(c1,c2,type="l", main=what);abline(0,1);arrows(c1[-length(c1)],c2[-length(c1)],c1[-1],c2[-1], length=0.05, col="blue")
   matplot(k, cbind(c1, c2), type="l", col=1:2, lty=1:2, ylab="ld(possibilities)/n", main=what)
   legend(par("usr")[1],par("usr")[3],c("exact.margin.info","approx.margin.info"), xjust=0, yjust=0, lty=1:2, col=1:2)
   i <- (n\%\%k)==0
   matpoints(k[i], cbind(c1[i],c2[i]), col=1:2, pch=1:2)

   what <- "compare at fixed k with increasing n"
   cat("\n", what, "\n")
   k <- 100
   n <- 100:2000
   g <- lapply(n, function(n){g <- rep(n\%/\%k, k);g[1]<-g[1]+n\%\%k;g})
   c1 <- sapply(g, exact.margin.info)
   c2 <- sapply(g, approx.margin.info)
   eqscplot(c1,c2,type="l", main=what);abline(0,1);arrows(c1[-length(c1)],c2[-length(c1)],c1[-1],c2[-1], length=0.05, col="blue")
   matplot(n, cbind(c1, c2), type="l", col=1:2, lty=1:2, ylab="ld(possibilities)/n", main=what)
   legend(par("usr")[1],par("usr")[3],c("exact.margin.info","approx.margin.info"), xjust=0, yjust=0, lty=1:2, col=1:2)

   what <- "compare at fixed k and n with increasing main category (rest outliers)"
   cat("\n", what, "\n")
   k <- 10
   n <- 10:1000
   g <- lapply(n, function(n){g <- c(rep(1, k-1), n-9)})
   c1 <- sapply(g, exact.margin.info)
   c2 <- sapply(g, approx.margin.info)
   eqscplot(c1,c2,type="l", main=what);abline(0,1);arrows(c1[-length(c1)],c2[-length(c1)],c1[-1],c2[-1], length=0.05, col="blue")
   matplot(n, cbind(c1, c2), type="l", col=1:2, lty=1:2, ylab="ld(possibilities)/n", main=what)
   legend(par("usr")[1],par("usr")[3],c("exact.margin.info","approx.margin.info"), xjust=0, yjust=0, lty=1:2, col=1:2)

   what <- "compare at fixed k and n with increasing main category (small groups)"
   cat("\n", what, "\n")
   k <- 10
   n <- 500:1500
   g <- lapply(n, function(n){g <- c(rep(50, k-1), n-450)})
   c1 <- sapply(g, exact.margin.info)
   c2 <- sapply(g, approx.margin.info)
   eqscplot(c1,c2,type="l", main=what);abline(0,1);arrows(c1[-length(c1)],c2[-length(c1)],c1[-1],c2[-1], length=0.05, col="blue")
   matplot(n, cbind(c1, c2), type="l", col=1:2, lty=1:2, ylab="ld(possibilities)/n", main=what)
   legend(par("usr")[1],par("usr")[3],c("exact.margin.info","approx.margin.info"), xjust=0, yjust=0, lty=1:2, col=1:2)

   what <- "compare at fixed k and n with increasing first category"
   cat("\n", what, "\n")
   k <- 3
   n <- 900
   n1 <- 1:300
   g <- lapply(n1, function(n1){g <- c(n1, rep((n-n1)\%/\%(k-1),k-1));g[2]<-g[2]+(n-n1)\%\%(k-1);g})
   c1 <- sapply(g, exact.margin.info)
   c2 <- sapply(g, approx.margin.info)
   eqscplot(c1,c2,type="l", main=what);abline(0,1);arrows(c1[-length(c1)],c2[-length(c1)],c1[-1],c2[-1], length=0.05, col="blue")
   matplot(n1, cbind(c1, c2), type="l", col=1:2, lty=1:2, ylab="ld(possibilities)/n", main=what)
   legend(par("usr")[1],par("usr")[3],c("exact.margin.info","approx.margin.info"), xjust=0, yjust=0, lty=1:2, col=1:2)
}
\keyword{ math }
\keyword{ cluster }
\name{extremegroup}
\docType{data}
\alias{extremegroup}
\title{Artificial dataset: extremegroup}
\description{
Two convex clusters with a third small very outlying cluster.
}
\usage{data(extremegroup)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\seealso{ \code{\link{SASmodeclus}}, \code{\link{hierarchic}} }
\keyword{datasets}
\name{flipper}
\docType{data}
\alias{flipper}
\title{Artificial dataset: flipper}
\description{
Four convex clusters without a pairwise grouping such that a 2 cluster solution is not stable.
}
\usage{data(flipper)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{noflipper}}, \code{\link{symflipper}} }
\keyword{datasets}
\name{hierarchic}
\docType{data}
\alias{hierarchic}
\title{Artificial dataset: hierarchic}
\description{
One big and two small clusters, where one of the small clusters has the same structure again. Thus both, a 3-cluster and a 5-cluster solution have some justification.
}
\usage{data(hierarchic)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\seealso{ \code{\link{conflict}}, \code{\link{noflipper}}, \code{\link{cophenetic}}  }
\keyword{datasets}
\name{margination}
\alias{labelation}
\alias{nlabelation}
\alias{lnlabelation}
\alias{ldlabelation}
\alias{margination}
\alias{nmargination}
\alias{lnmargination}
\alias{ldmargination}
\title{ Margin combinations }
\description{
  Functions to calculate the number of possible groupings given fixed group sizes (margins).
}
\usage{
nlabelation(...)
lnlabelation(...)
ldlabelation(...)
nmargination(...)
lnmargination(...)
ldmargination(...)
}
\arguments{
  \item{\dots}{ one or more vectors of group sizes }
}
\details{
  Function \code{nmargination} gives the number of possibles groupings (=cluster solutions) that have the input group sizes.
  Function \code{nlabelation} gives the number of possibles assignents to predefined (=labelled) groups of the requires sizes (more possibilities).
  The ln-versions give the log instead, the ld-versions the logarithm dualis.
}
\value{
  scalar
}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{clusteration}}, \code{\link{permutation}}, \code{\link{combination}}, \code{\link[base]{choose}}, \code{\link[base]{lgamma}}, \code{\link{absolute.entropy}}, \code{\link{exact.margin.info}} }
\examples{
  cat("There are 25 possibilities to make 3 clusters from 5 cases\n")
  nclusteration(5,3)
  cat(" 10 of sizes 3,1,1\n")
  nmargination(3,1,1)
  cat(" and 15 of sizes 2,2,1\n")
  nmargination(2,2,1)
  cat("By contrast there are 30 possibilities to assign 5 cases to 3 labelled groups a=2,b=2,c=1\n")
  nlabelation(2,2,1)
  n <- 250
  k <- 5
  i <- 1:(n/k)
  matplot(k*i, cbind(
      k^(i*k)
    , sapply(i, function(i){nclusteration(k*i, k)})
    , sapply(i, function(i){nmargination(rep(i, k))})
    )
  , log="y", type="l"
  , lty=1:3, xlab="n"
  , ylab = "combinations"
  , main = "number of possible groupings"
  )
  legend(n,1
  , c(substitute(k^n, list(k=k)), expression("no restriction"), expression("equal sizes"))
  , lty=1:3, col=1:3, xjust=1, yjust=0
  )
}
\keyword{ math }
\keyword{ cluster }
\name{matchindex}
\alias{matchindex}
\alias{matchtable}
\alias{matchcalcs}
\title{ Matching for cross-tables }
\description{
  Permutes rows and columns of a cross-table to match the marginal classes.
}
\usage{
matchindex(observed, method = c("heuristic", "truematch", "tracemax")[1])
matchtable(observed, method = c("heuristic", "truematch", "tracemax")[1])
matchcalcs(k)
}
\arguments{
  \item{observed}{ an integer matrix representing a cross-table, non-square allowed }
  \item{method}{ one of c("heuristic","truematch","tracemax"), see details }
  \item{k}{ number of categories to match }
}
\details{
  Method \emph{heuristic} -- the default -- matches using the truematch heuristic, i.e. recursively calculating resisuals and removing the row/col with the biggest residuals. The number of residuals calculated during matching is given by function \code{matchcalcs}  = 1/6*(n*(n+1)*(2*n+1)) - 1 (Otto, Forster, Analysis 1, Diffential und Integralrechnung einer Vernderlichen, Viehweg, Braunschweig (1992).). Thus the truematch heuristic has time-complexity O(n^3), and memory-complexity O(n^2), if replacing recursion by while loops as done here.
  Method \emph{truematch} -- the truematch algorithm -- calculates residuals just once and then applies trace maximization (Munkres Hungarian Method).
  Method \emph{tracemax} applies trace maximization directly without calculating residuals.
  All methods can cope with (non-square) rectangular matrices. All methods break ties by random. All methods use C implementations. All methods have polynomial time complexity.
}
\value{
  Function \code{matchindex} returns a list with components
  \item{row}{permutation index for the rows}
  \item{col}{permutation index for the columns}
  Function \code{matchtable} returns reordered table
  \item{row}{permutation index for the rows}
  \item{col}{permutation index for the columns}
  Function \code{matchcalcs} returns the number of residuals to calculate during matching
}
\references{ xx }
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{munkres}}, for another implementation  of the Hungarian method see \code{\link[clue]{solve_LSAP}}, for greedy heuristics see \code{\link[e1071]{matchClasses}} }

\examples{
m <- rbind(c(1, 98), c(0, 1))
i <- matchindex(m)
i
m[i$row, i$col]
i <- matchindex(m, method="truematch")
i
m[i$row, i$col]
i <- matchindex(m, method="tracemax")
i
m[i$row, i$col]
  \dontrun{
     library(e1071)
     library(clue)
     #library(lpSolve)

     cat("Check quality with respect to trace maximization\n")
     x <- integer(6)
     names(x) <- c("e","s","l","t","r","m")
     for (i in 1:400){
        k <- 5
        m <- table(sample(1:k, 10000, TRUE), sample(1:k, 10000, TRUE))
        i.e <- matchClasses(m, method="exact")
        m.e <- m[,i.e]
        i.s <- solve_LSAP(m, maximum=TRUE)
        m.s <- m[,i.s]
        i.l <- apply(lp.assign(-m)$solution, 1, which.max)
        m.l <- m[,i.l]
        i.t <- matchindex(m, method="tracemax")
        m.t <- m[i.t$row, i.t$col]
        i.r <- matchindex(m, method="truematch")
        m.r <- m[i.r$row, i.r$col]
        i.m <- matchindex(m, method="truecluster")
        m.m <- m[i.m$row, i.m$col]
        x["e"] <- x["e"] + sum(diag(m.e))
        x["s"] <- x["s"] + sum(diag(m.s))
        x["l"] <- x["l"] + sum(diag(m.l))
        x["t"] <- x["t"] + sum(diag(m.t))
        x["r"] <- x["r"] + sum(diag(m.r))
        x["m"] <- x["m"] + sum(diag(m.m))
     }
     x / max(x)

     cat("Check speed\n")
     K <- 2^(2:9)
     n <- length(K)
     tim <- matrix(NA, nrow=n, ncol=3, dimnames=list(NULL, c("s","m","t")))
     for (i in 1:n){
        k <- K[i]
        m <- table(sample(1:k, 1000000, TRUE), sample(1:k, 1000000, TRUE))
        tim[i,"s"] <- system.time({solve_LSAP(m)})[3]
        tim[i,"m"] <- system.time({matchindex(m, method="truecluster")})[3]
        tim[i,"t"] <- system.time({matchindex(m, method="tracemax")})[3]
        matplot(K, tim, pch=c("s","m","t"))
     }
  }
}
\keyword{ utilities }
\name{mmcc2mmcp}
\alias{mmcc2mmcp}
\alias{mmcc2cluster}
\alias{mmcp2pk}
\alias{mmcp2RMC}
\alias{mmcp2h}
\alias{mmcp2H}
\alias{mmcp2d}
\alias{mmcp2D}
\alias{mmcp2i}
\alias{mmcp2I}
\alias{mmcp2cic}
\alias{mmcp2CIC}
\alias{mmcp2ALL}
\alias{mmcp2gsd}
\alias{mmcp2GSD}
\alias{cic}
\alias{CIC}
\alias{mmcc}
\alias{MMCC}
\alias{gsd}
\alias{GSD}
\title{MMCC Evaluation}
\description{
  These functions evaluate the MMCC matrix created by resample aggregation.
}
\usage{
mmcc2cluster(c)
mmcc2mmcp(c, rowsums = NULL, equal.rowsums = FALSE)
mmcp2pk(p)   # marginal cluster probabilities
mmcp2RMC(p)  # relative model complexity
mmcp2h(p)    # cellwise conditional entropy = uncertainty
mmcp2H(p)    # conditional entropy = uncertainty
mmcp2d(p)    # cellwise non-penalized information
mmcp2D(p)    # non-penalized information
mmcp2i(p)    # cellwise penalized information
mmcp2I(p)    # penalized information
mmcp2cic(p)  # cellwise CIC
mmcp2CIC(p)  # CIC
mmcp2ALL(p)  # c(N, H, D, RMC, I, CIC)
mmcp2gsd(p)  # rowwise Generalized Silhouette Value
mmcp2GSD(p, FUN=summary)  # clusterwise statistics of Generalized Silhouette Value
}
\arguments{
  \item{c}{MMCC matrix of cluster votes}
  \item{p}{MMCP matrix of cluster probabilities}
  \item{rowsums}{specifying vector or scalar of rowsums of mmcc speeds up calculations}
  \item{equal.rowsums}{TRUE uses sum(mmcc[1,]) as rowsums, ignored if rowsums is given }
  \item{FUN}{a function summarizing a vector of GSD values (called once per cluster)}
}
\details{
 \code{mmcc2cluster} assigns each case (row) to the most likely cluster (column) breaking ties at random.
}
\value{
  Function mmcc2cluster returns a vector of cluster memberships.
  Function mmcc2mmcp returns converts an MMCC (counted cluster votes) matrix into an MMCP matrix (estimated cluster probabilities).
  Function mmcp2RMC returns scalar giving the \emph{relative model complexity}
  Function mmcp2gsd returns a vector of Generalized Silhouette Diagnostics
  Function mmcp2GSD returns a matrix[statistics,clusters] summarizing Generalized Silhouette Diagnostics
  Function mmcp2xx returns a matrix derived from MMCP.
  Function mmcp2XX returns mean(rowSums(mmcp2xx())) and might be a bit faster.
}
\references{ xx }
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{twoway.entropy}}, \code{\link{ldmargination}} }
\examples{
cat("\nfake a cluster voting matrix\n")
c <- c(rbinom(4, 256, 0.25), rbinom(12, 256, 0.75))
c <- cbind(a=c, b=256-c)
c
mmcc2cluster(c)
table(mmcc2cluster(c))
cat("\nconvert to probabilities matrix\n")
p <- mmcc2mmcp(c)
p

cat("\nlook at Relative Model Complexity\n")
mmcp2RMC(p)

cat("\ncellwise Cluster Information Criterion\n")
mmcp2cic(p)
cat("\ncluster CIC\n")
colSums(mmcp2cic(p))
cat("\ncase CIC (aggregated by cluster)\n")
aggregate(rowSums(mmcp2cic(p)), by=list(mmcc2cluster(p)), FUN=mean)
cat("\ncalculating model CIC\n")
mean(rowSums(mmcp2cic(p)))
mmcp2CIC(p)
cat("\ncalculating all model statistics\n")
mmcp2ALL(p)
cat("\nGeneralized Silhouette Diagnostic\n")
mmcp2GSD(p)
boxplot(split(mmcp2gsd(p), mmcc2cluster(p)), main="GSD distribution by cluster")
boxplot(split(rowSums(mmcp2h(p)), mmcc2cluster(p)), main="Uncertainty distribution by cluster")

cat("\nshow that ties are broken at random\n")
c <- matrix(1, 16, 4)
table(mmcc2cluster(c))
table(mmcc2cluster(c))

cat("\ncompare some models differing in true and model complexity\n")
collect <- NULL
c <- matrix(1, 16, 1)
collect <- rbind(collect, "true=1  model=1"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=1  model=2"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=1  model=4"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=1  model=8"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=1  model=16"=mmcp2ALL(mmcc2mmcp(c)))
c <- t(matrix(rep(diag(2), 8), 2))
collect <- rbind(collect, "true=2  model=2"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=2  model=4"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=2  model=8"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=2  model=16"=mmcp2ALL(mmcc2mmcp(c)))
c <- t(matrix(rep(diag(4), 4), 4))
collect <- rbind(collect, "true=4  model=4"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=4  model=8"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=4  model=16"=mmcp2ALL(mmcc2mmcp(c)))
c <- t(matrix(rep(diag(8), 2), 8))
collect <- rbind(collect, "true=8  model=8"=mmcp2ALL(mmcc2mmcp(c)))
c <- cbind(c, c)
collect <- rbind(collect, "true=8  model=16"=mmcp2ALL(mmcc2mmcp(c)))
c <- diag(16)
collect <- rbind(collect, "true=16 model=16"=mmcp2ALL(mmcc2mmcp(c)))
colnames(collect)[c(5,6)] <- c("D*(1-RMC)=I","I-H=CIC")
print(collect[order(collect[,6]),])

}
\keyword{multivariate}
\keyword{cluster}
\name{munkres}
\alias{munkres}
\title{ Hungarian method for rectangular cost matrices }
\description{
  Rows and columns of a cost matrix are matched such that the trace costs are minimized.
}
\usage{
  munkres(x, tieorder = TRUE, tiebreak = TRUE, decreasing = FALSE)
}
\arguments{
  \item{x}{ numeric cost matrix }
  \item{tieorder}{ default TRUE to order the diagonal }
  \item{tiebreak}{ default TRUE to break ties at random when ordering the diagonal}
  \item{decreasing}{ TRUE to order decreasing (default FALSE) }
}
\details{
  The is a version of Munkres' hungarian method, which allows for non-square rectangular cost matrices.
}
\value{
  A list
  \item{row}{index for reordering the rows}
  \item{col}{index for reordering the columns}
}
\references{ xx }
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{tieorder}}, \code{\link{matchindex}}, \code{\link[clue]{solve_LSAP}}, \code{\link[e1071]{matchClasses}} }

\examples{
 rows <- 4
 cols <- 6
 mat <- matrix(runif(rows*cols), nrow=rows, ncol=cols)
 cat("find the matching minimizing costs\n")
 ind <- munkres(mat); mat[ind$row, ind$col]
 cat("the same with decreasing order\n")
 ind <- munkres(mat, decreasing=TRUE); mat[ind$row, ind$col]
 cat("find the matchings maximizing costs\n")
 ind <- munkres(-mat); mat[ind$row, ind$col]
}
\keyword{ utilities }
\name{noflipper}
\docType{data}
\alias{noflipper}
\title{Artificial dataset: noflipper}
\description{
Four convex clusters with a pairwise grouping such that a 2 cluster solution is stable as well.
}
\usage{data(noflipper)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{flipper}}, \code{\link{symflipper}}, \code{\link{hierarchic}} }
\keyword{datasets}
\name{permutation}
\alias{permutation}
\alias{npermutation}
\title{ Permutations }
\description{
  Calculating and creating permutations
}
\usage{
npermutation(k, n, replace = FALSE)
permutation(k, n=NULL, x=NULL, replace = FALSE)
}
\arguments{
  \item{k}{ number of positions }
  \item{n}{ poolsize }
  \item{x}{ vector pool of values, can be given instead of n }
  \item{replace}{ default FALSE, TRUE to allow reusing of pool elements }
}
\details{
  \code{npermutation} returns the number of permutations, \code{permutation} returns a matrix of permutations. No special sequence granted. Implemented in C, but not faster than the functions in gregmisc.
}
\value{
  A matrix of (k x npermutation)
}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{combination}}, \code{\link{margination}}, \code{\link[base]{expand.grid}}, \code{\link[gregmisc]{combinations}} }
\examples{
  npermutation(2,4)
  permutation(2,4)
  permutation(2,x=letters[1:4])
  npermutation(2,4,replace=TRUE)
  permutation(2,4,replace=TRUE)

  \dontrun{
  ## regression test (error in gregmisc for 1.5.1)
  require(gregmisc)
  k <- 3
  n <- 6
  replace <- FALSE
  a <- sort(apply(permutation(k,n,replace=replace), 2, paste, collapse="-"))
  b <- sort(apply(permutations(n,k,rep=replace), 1, paste, collapse="-"))
  stopifnot(length(a)==npermutation(k,n,replace=replace))
  stopifnot(identical(a,b))
  replace <- TRUE
  a <- sort(apply(permutation(k,n,replace=replace), 2, paste, collapse="-"))
  b <- sort(apply(permutations(n,k,rep=replace), 1, paste, collapse="-"))
  stopifnot(length(a)==npermutation(k,n,replace=replace))
  stopifnot(identical(a,b))
  }
}
\keyword{ iteration }
\name{shannon.information}
\alias{laplace.probability}
\alias{laplace.information}
\alias{shannon.information}
\alias{absolute.entropy}
\alias{normalized.entropy}
\alias{redundancy}
\alias{distribution.entropy}
\alias{density.entropy}
\title{Univariate Informationtheoretic Measures}
\description{
  These functions calculate univariate informationtheoretic measures, like \emph{information}, \emph{entropy} and \emph{redundancy}.
}
\usage{
laplace.probability(n)
laplace.information(n)
shannon.information(p)
absolute.entropy(p)
normalized.entropy(p)
redundancy(p)
distribution.entropy(x, FUN = pnorm, ..., normalized = TRUE)
density.entropy(lower = -Inf, upper = Inf, FUN = dnorm, ..., normalized = TRUE)
}
\arguments{
  \item{n}{ scalar: number of (equal likely) possibilities }
  \item{p}{ vector of probabilities }
  \item{x}{ sequence of values at which FUN is evaluated }
  \item{lower}{ lower bound for integrate }
  \item{upper}{ upper bound for integrate }
  \item{FUN}{ a cumulative distribution function }
  \item{\dots}{ further arguments to FUN }
  \item{normalized}{ FALSE to use \code{absolute.entropy} instead of TRUE (default) \code{normalized.entropy} }
}
\value{
  Function \code{shannon.information} returns a vector of log2 (or zero where p=0). Entropy functions return vector of entropy contributions for each p.
}
\references{ MacKay, David J.C. (2003). \emph{Information Theory, Inference, and Learning Algorithms (chapter 8).} Cambridge University Press.}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{twoway.entropy}}, \code{\link[distutil]{dist.entropy}}, \code{\link{Kullback.Leibler}}, \code{\link[base]{log}}, \code{\link{exact.margin.info}} }
\examples{

  # elementary
  laplace.information(2)      # = log2(n)
  p <- laplace.probability(2) # = rep(1/n, n)
  p
  shannon.information(p)      # = log2(p)
  absolute.entropy(p)         # = p*log2(p)
  normalized.entropy(p)       # = p*log2(p) / log2(n)

  sum(absolute.entropy(p))    # Max = log2(n)
  sum(normalized.entropy(p))  # Max = 1
  sum(redundancy(p))          # Redundancy = 1 - NormalizedEntropy

  laplace.information(2)
  laplace.information(4)  # more categories, more information
  sum(absolute.entropy(laplace.probability(2)))
  sum(absolute.entropy(laplace.probability(4)))   # more categories, more entropy
  sum(normalized.entropy(laplace.probability(2)))
  sum(normalized.entropy(laplace.probability(4))) # more categories, constant normalized entropy
  p <- c(0.5, 0.25, 0.15, 0.1)
  sum(normalized.entropy(p))                      # unequal probabilities, lower entropy (normalized or not)

  sum(distribution.entropy(seq(-3, 3, 0.01)))
  sum(distribution.entropy(seq(-3, 3, 0.001)))
  sum(distribution.entropy(seq(-30, 30, 0.01), sd=10))
  sum(distribution.entropy(seq(-3, 3, 0.01), FUN=punif, -3, 3))

  sum(distribution.entropy(seq(-3, 3, 0.01), normalized=FALSE))
  sum(distribution.entropy(seq(-3, 3, 0.001), normalized=FALSE))
  sum(distribution.entropy(seq(-30, 30, 0.01), sd=10, normalized=FALSE))
  sum(distribution.entropy(seq(-3, 3, 0.01), FUN=punif, -3, 3, normalized=FALSE))

  density.entropy(-3, 3)
  density.entropy(-30, 30, sd=10)

}
\keyword{ univar }
\name{spiral}
\docType{data}
\alias{spiral}
\title{Artificial dataset: spiral}
\description{
Two spiral-shaped clusters occupying the same topological area. Can be detected by density based cluster algorithms or single link.
}
\usage{data(spiral)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{centerring}}, \code{\link{SASmodeclus}} }
\keyword{datasets}
\name{symflipper}
\docType{data}
\alias{symflipper}
\title{Artificial dataset: symflipper}
\description{
Four convex clusters touching each other, double symmetry prevents unique 2-cluster solution.
}
\usage{data(symflipper)}
\format{A dataframe with columns x, y and cluster}
\source{Artificial}
\references{
Oehlschlgel, J. (2006) \emph{Scalable robust clustering with model selection}.
}
\seealso{ \code{\link{flipper}}, \code{\link{noflipper}} }
\keyword{datasets}
\name{tieorder}
\alias{tieorder}
\title{ Ordering double breaking ties at random }
\description{
  This function works very similar to \code{\link[base]{order}} but in case of ties it breaks them by random.
}
\usage{
tieorder(x, decreasing = FALSE, index = seq(along = x), tiebreak = TRUE)
}
\arguments{
  \item{x}{ a vector of double or integer }
  \item{decreasing}{ TRUE to order decreasing (default FALSE = increasing) }
  \item{index}{ optionally a vector of integer index to be rearranged }
  \item{tiebreak}{ FALSE to avoid randomly breaking ties }
}
\details{
  NAs sorted last. Uses internally rsort_index.
}
\value{
  index rearranged
}
\author{ Jens Oehschlgel }
\seealso{ \code{\link[base]{order}}, \code{\link[distutil]{colOrder}}, \code{\link[distutil]{rowOrder}} }

\examples{
  tieorder(c(NA, NA, 2, 2, 1))
  tieorder(c(NA, NA, 2, 2, 1))
  tieorder(c(NA, NA, 2, 2, 1))
  tieorder(c(NA, NA, 2, 2, 1))
  x <- c(NA, NA, 2, 2, 1)
  n <- 100
  o <- integer(length(x))
  for (i in 1:n)
  o <- o + tieorder(x)
  o/n
}
\keyword{ utilities }
\name{twoway.entropy}
\alias{twoway.entropy}
\alias{print.twoway.entropy}
\alias{plot.twoway.entropy}
\title{Bivariate Informationtheoretic Measures}
\description{
  These functions calculate bivariate informationtheoretic measures, like \emph{joint entropy}, \emph{conditional entropy} and \emph{mutual information}.
}
\usage{
twoway.entropy(p, grain=c("Totals","Margins","Cells"))
print.twoway.entropy(x, ...)
plot.twoway.entropy(x, ...)
}
\arguments{
  \item{p}{ matrix of two-way probabilities (or frequencies) }
  \item{x}{ on object of class twoway.entropy }
  \item{grain}{ one of c("Totals","Margins","Cells") determining the granularity of the return value }
  \item{\dots}{ further arguments to plot,print }
}
\value{
  An object of class twoway.entropy with components
  \item{Totals}{always returned: a list with components \emph{H} joint entropy, \emph{Ha} row entropy, \emph{Hab}  row conditional entropy given columns, \emph{Hb} column entropy, \emph{Hba} column conditional entropy given rows, \emph{Im} mutual information }
  \item{Margins}{returned unless grain="Total": \emph{pa} row probabilities , \emph{ha} row entropie, \emph{hab} colwise conditional entropies, \emph{pb} column probabilities, \emph{hb} column entropy , \emph{hba} rowwise conditional entropies}
  \item{Cells}{returned if grain="Cells": \emph{p} joint probabilities, \emph{pab} columnwise conditional probabilities, \emph{pba} rowwise conditional probabilities, \emph{h} joint entropies, \emph{hab} columnwise conditional entropies, \emph{hba} rowwise conditional entropies }
}
\references{ MacKay, David J.C. (2003). \emph{Information Theory, Inference, and Learning Algorithms (chapter 8).} Cambridge University Press.}
\author{ Jens Oehlschlgel }
\seealso{ \code{\link{shannon.information}}, \code{\link[distutil]{dist.entropy}}, \code{\link{Kullback.Leibler}}, \code{\link[base]{log}} }
\examples{
 # Exercise 8.6 from MacKay
 p <- 1/matrix(c(8,16,16,4,16,8,16,Inf,32,32,16,Inf,32,32,16,Inf), 4)
 twoway.entropy(p)
 twoway.entropy(p, grain="Cells")
 str(twoway.entropy(p, grain="Cells"))
 plot(twoway.entropy(p), main="Plot from MacKay (2003), chapter 8.1")

 # Highly correlated data
 x <- rnorm(100)
 y <- x + rnorm(100, sd=0.1)
 a <- round(x)
 b <- round(y)
 p <- table(a, b)
 plot(twoway.entropy(p), main=paste("correlation=", round(cor(x,y), 2)))
}
\keyword{ multivariate }
